@@ -4,7 +4,7 @@ declare(strict_types=1);
 
 namespace Doctrine\Tests\ORM\Functional\Ticket;
 
-use Doctrine\ORM\Mapping\ClassMetadataInfo;
+use Doctrine\ORM\Mapping\ClassMetadata;
 use Doctrine\Tests\Models;
 use Doctrine\Tests\OrmFunctionalTestCase;
 
@@ -23,9 +23,9 @@ class DDC1301Test extends OrmFunctionalTestCase
         parent::setUp();
 
         $class                                             = $this->_em->getClassMetadata(Models\Legacy\LegacyUser::class);
-        $class->associationMappings['articles']['fetch']   = ClassMetadataInfo::FETCH_EXTRA_LAZY;
-        $class->associationMappings['references']['fetch'] = ClassMetadataInfo::FETCH_EXTRA_LAZY;
-        $class->associationMappings['cars']['fetch']       = ClassMetadataInfo::FETCH_EXTRA_LAZY;
+        $class->associationMappings['articles']['fetch']   = ClassMetadata::FETCH_EXTRA_LAZY;
+        $class->associationMappings['references']['fetch'] = ClassMetadata::FETCH_EXTRA_LAZY;
+        $class->associationMappings['cars']['fetch']       = ClassMetadata::FETCH_EXTRA_LAZY;
 
         $this->loadFixture();
     }
@@ -35,15 +35,15 @@ class DDC1301Test extends OrmFunctionalTestCase
         parent::tearDown();
 
         $class                                             = $this->_em->getClassMetadata(Models\Legacy\LegacyUser::class);
-        $class->associationMappings['articles']['fetch']   = ClassMetadataInfo::FETCH_LAZY;
-        $class->associationMappings['references']['fetch'] = ClassMetadataInfo::FETCH_LAZY;
-        $class->associationMappings['cars']['fetch']       = ClassMetadataInfo::FETCH_LAZY;
+        $class->associationMappings['articles']['fetch']   = ClassMetadata::FETCH_LAZY;
+        $class->associationMappings['references']['fetch'] = ClassMetadata::FETCH_LAZY;
+        $class->associationMappings['cars']['fetch']       = ClassMetadata::FETCH_LAZY;
     }
 
     public function testCountNotInitializesLegacyCollection(): void
     {
-        $user       = $this->_em->find(Models\Legacy\LegacyUser::class, $this->userId);
-        $queryCount = $this->getCurrentQueryCount();
+        $user = $this->_em->find(Models\Legacy\LegacyUser::class, $this->userId);
+        $this->getQueryLog()->reset()->enable();
 
         self::assertFalse($user->articles->isInitialized());
         self::assertCount(2, $user->articles);
@@ -52,13 +52,13 @@ class DDC1301Test extends OrmFunctionalTestCase
         foreach ($user->articles as $article) {
         }
 
-        self::assertEquals($queryCount + 2, $this->getCurrentQueryCount(), 'Expecting two queries to be fired for count, then iteration.');
+        $this->assertQueryCount(2, 'Expecting two queries to be fired for count, then iteration.');
     }
 
     public function testCountNotInitializesLegacyCollectionWithForeignIdentifier(): void
     {
-        $user       = $this->_em->find(Models\Legacy\LegacyUser::class, $this->userId);
-        $queryCount = $this->getCurrentQueryCount();
+        $user = $this->_em->find(Models\Legacy\LegacyUser::class, $this->userId);
+        $this->getQueryLog()->reset()->enable();
 
         self::assertFalse($user->references->isInitialized());
         self::assertCount(2, $user->references);
@@ -67,13 +67,13 @@ class DDC1301Test extends OrmFunctionalTestCase
         foreach ($user->references as $reference) {
         }
 
-        self::assertEquals($queryCount + 2, $this->getCurrentQueryCount(), 'Expecting two queries to be fired for count, then iteration.');
+        $this->assertQueryCount(2, 'Expecting two queries to be fired for count, then iteration.');
     }
 
     public function testCountNotInitializesLegacyManyToManyCollection(): void
     {
-        $user       = $this->_em->find(Models\Legacy\LegacyUser::class, $this->userId);
-        $queryCount = $this->getCurrentQueryCount();
+        $user = $this->_em->find(Models\Legacy\LegacyUser::class, $this->userId);
+        $this->getQueryLog()->reset()->enable();
 
         self::assertFalse($user->cars->isInitialized());
         self::assertCount(3, $user->cars);
@@ -82,7 +82,7 @@ class DDC1301Test extends OrmFunctionalTestCase
         foreach ($user->cars as $reference) {
         }
 
-        self::assertEquals($queryCount + 2, $this->getCurrentQueryCount(), 'Expecting two queries to be fired for count, then iteration.');
+        $this->assertQueryCount(2, 'Expecting two queries to be fired for count, then iteration.');
     }
 
     public function loadFixture(): void
@@ -90,17 +90,14 @@ class DDC1301Test extends OrmFunctionalTestCase
         $user1           = new Models\Legacy\LegacyUser();
         $user1->username = 'beberlei';
         $user1->name     = 'Benjamin';
-        $user1->_status  = 'active';
 
         $user2           = new Models\Legacy\LegacyUser();
         $user2->username = 'jwage';
         $user2->name     = 'Jonathan';
-        $user2->_status  = 'active';
 
         $user3           = new Models\Legacy\LegacyUser();
         $user3->username = 'romanb';
         $user3->name     = 'Roman';
-        $user3->_status  = 'active';
 
         $this->_em->persist($user1);
         $this->_em->persist($user2);
